@@ -2,6 +2,7 @@ from __future__ import unicode_literals
 
 from collections import defaultdict
 
+from django.conf import settings
 from django.contrib import messages
 from django.utils.translation import ugettext as _
 from django.utils.encoding import force_text
@@ -42,8 +43,9 @@ class FSMTransitionMixin(object):
     #      _fsmtransition-revision_state-delete
     fsm_input_prefix = '_fsmtransition'
     # The name of one or more FSMFields on the model to transition
-    fsm_field = ['state',]
+    fsm_field = ['state']
     change_form_template = 'fsm_admin/change_form.html'
+    default_disallow_transition = not getattr(settings, 'FSM_ADMIN_FORCE_PERMIT', False)
 
     def _fsm_get_transitions(self, obj, request, perms=None):
         """
@@ -58,8 +60,8 @@ class FSMTransitionMixin(object):
         transitions = {}
         for field in fsm_fields:
             transitions_func = 'get_available_user_{0}_transitions'.format(field)
-            transitions[field] = getattr(obj, transitions_func)(user) if obj else []
-
+            transitions_generator = getattr(obj, transitions_func)(user) if obj else []
+            transitions[field] = self._filter_admin_transitions(transitions_generator)
         return transitions
 
     def get_redirect_url(self, request, obj):
@@ -73,7 +75,7 @@ class FSMTransitionMixin(object):
         Returns the actual state field instance, as opposed to
         fsm_field attribute representing just the field name.
         """
-        return self.model._meta.get_field_by_name(fsm_field_name)[0]
+        return self.model._meta.get_field(fsm_field_name)
 
     def display_fsm_field(self, obj, fsm_field_name):
         """
@@ -112,9 +114,28 @@ class FSMTransitionMixin(object):
         Checks if the requested transition is available
         """
         transitions = []
-        for field, field_transitions in self._fsm_get_transitions(obj, request).iteritems():
+        for field, field_transitions in iter(self._fsm_get_transitions(obj, request).items()):
             transitions += [t.name for t in field_transitions]
-        return transitions
+        return transition in transitions
+
+    def _filter_admin_transitions(self, transitions_generator):
+        """
+        Filter the given list of transitions, if their transition methods are declared as admin
+        transitions. To allow a transition inside fsm_admin, add the parameter
+        `admin=True` to the transition decorator, for example:
+        ```
+        @transition(field='state', source=['startstate'], target='finalstate', custom=dict(admin=True))
+        def do_something(self):
+            ...
+        ```
+
+        If the configuration setting `FSM_ADMIN_FORCE_PERMIT = True` then only transitions with
+        `custom=dict(admin=True)` are allowed. Otherwise, if `FSM_ADMIN_FORCE_PERMIT = False` or
+        unset only those with `custom=dict(admin=False)`
+        """
+        for transition in transitions_generator:
+            if transition.custom.get('admin', self.default_disallow_transition):
+                yield transition
 
     def _get_requested_transition(self, request):
         """
@@ -139,11 +160,15 @@ class FSMTransitionMixin(object):
         if available and trans_func:
             # Run the transition
             try:
-                # Attempt to pass in the by argument if using django-fsm-log
-                trans_func(by=request.user)
+                # Attempt to pass in the request and by argument if using django-fsm-log
+                trans_func(request=request, by=request.user)
             except TypeError:
-                # If the function does not have a by attribute, just call with no arguments
-                trans_func()
+                try:
+                    # Attempt to pass in the by argument if using django-fsm-log
+                    trans_func(by=request.user)
+                except TypeError:
+                    # If the function does not have a by attribute, just call with no arguments
+                    trans_func()
             new_state = self.display_fsm_field(obj, fsm_field_name)
 
             # Mark the fsm_field as changed in the form so it will be
@@ -180,9 +205,17 @@ class FSMTransitionMixin(object):
                 if condition(obj):
                     continue
 
+                # if the transition is hidden, we don't need the hint
+                if transition.custom.get('admin', self.default_disallow_transition):
+                    continue
+
                 hint = getattr(condition, 'hint', '')
                 if hint:
-                    hints[transition.name].append(hint)
+                    if hasattr(transition, 'custom') and transition.custom.get(
+                            'button_name'):
+                        hints[transition.custom['button_name']].append(hint)
+                    else:
+                        hints[transition.name.title()].append(hint)
 
         return dict(hints)
 
@@ -192,7 +225,7 @@ class FSMTransitionMixin(object):
         """
         fsm_fields = self._get_fsm_field_list()
         for field in fsm_fields:
-            fsmfield = obj._meta.get_field_by_name(field)[0]
+            fsmfield = obj._meta.get_field(field)
             transitions = fsmfield.get_all_transitions(self.model)
             for transition in transitions:
                 if transition.source in [getattr(obj, field), '*']:
@@ -206,6 +239,6 @@ class FSMTransitionMixin(object):
         accessing the property.
         """
         if not isinstance(self.fsm_field, (list, tuple,)):
-            return [self.fsm_field,]
+            return [self.fsm_field]
 
         return self.fsm_field
